/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.ivy.osgi.repo;

import java.util.ArrayList;
import java.util.List;

public class AggregatedOSGiResolver extends AbstractOSGiResolver {

    private List<AbstractOSGiResolver> resolvers = new ArrayList<AbstractOSGiResolver>();

    public void add(AbstractOSGiResolver resolver) {
        resolvers.add(resolver);
    }

    @Override
    protected void init() {
        List<RepoDescriptor> repos = new ArrayList<RepoDescriptor>();
        for (AbstractOSGiResolver resolver : resolvers) {
            repos.add(resolver.getRepoDescriptor());
        }
        setRepoDescriptor(new AggregatedRepoDescriptor(repos));
    }

}
