/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.ivy.core.pack;

import java.util.HashMap;
import java.util.Map;

public class PackingRegistry {

    private Map<String, ArchivePacking> packings = new HashMap<String, ArchivePacking>();

    public PackingRegistry() {
        // register defaults
        register(new ZipPacking());
        register(new Pack200Packing());
        register(new OsgiBundlePacking());
    }

    public void register(ArchivePacking packing) {
        for (String name : packing.getNames()) {
            packings.put(name, packing);
        }
    }

    public ArchivePacking get(String type) {
        return packings.get(type);
    }
}
